function CIR2
% In this exercise we simulated N paths of a CIR process
% dX_t = \lambda (\nu-X_t) dt + \sigma \sqrt(X_t) dW_t, X_0 =x and simulate the
% expecation of X_1, X_1^2, X_1^+ by generating non-central chi^2 random
% variables.
tic
%% parameter input
% horizon
T=1;
% sample size
Nsimu=10^4;
Nplot=Nsimu;
% grid points
M=10^3;
% volatility
lambda=1;
nu=1.2;
sigma=0.3;
x=1;
% time step
dt= T/M;

% Check the Feller condition
check = 2*lambda*nu >= sigma^2;
if check == 1
    disp('Feller condition is assumed')
elseif check==0
    disp('Feller condition is not satisfied')
end


% theoretical value for the expectation, second moment and pos part
constc = 4*lambda/(sigma^2*(1-exp(-lambda*T)));
constv = 4*lambda*nu/(sigma^2);
constlambda= constc*x*exp(-lambda*T);
% constc_T*X_T is non central chi square distributed (constv, constlambda)
% E(non-central chisquare distribution) = constv+constlambda;
%theoreticalvalueexp= (constv+constlambda)/constc;
theoreticalvalueexp= x*exp(-lambda*T)+nu*(1-exp(-lambda*T));
% Var(non-central chisquare distribution) = 2(constv+2*constlambda)
%theoreticalvaluesec= (2*(constv+2*constlambda)+(constv+constlambda)^2)/(constc^2);
theoreticalvaluesec= x*sigma^2/lambda*(exp(-lambda*T)-exp(-2*lambda*T))+nu*sigma^2/(2*lambda)*(1-exp(-lambda*T))^2+theoreticalvalueexp^2;
% Use numerical integration to obtain a theoretical value
% Since the Feller condition is assumed, E[X^+_1] must be equal to E[X_1]
theoreticalvaluepos=integral(@(x) x.*ncx2pdf(x,constv,constlambda),0,Inf)/constc;
%% Simulation
% parameters
% condition on X_s, c_t*X_t is noncentral chisq distributed with dof=
% constv and scale parameter = 4
% lambda/(sigma^2*(1-exp(-lambda*dt))*X_s*exp(1-lambda*dt)
dof = constv;
CIR = [x*ones(1,Nplot);zeros(M,Nplot)];
for i =1:M
    CIR(i+1,:)=ncx2rnd(dof*ones(1,Nplot),4*lambda/(sigma^2*(1-exp(-lambda*T/M)))*CIR(i,:)*exp(-lambda*T/M),[1,Nplot])/(4*lambda/(sigma^2*(1-exp(-lambda*T/M))));
end

%plot the first 10 sample paths
timegrid= 0:dt:T;
plot(timegrid,CIR(:,1:10))

%compute simulated value
simulatedvalueexp= mean(CIR(end,:));
simulatedvaluesec= mean(CIR(end,:).^2);
simulatedvaluepos= mean(subplus(CIR(end,:)));

disp('Exact values: Expectation/2.Moment/pos. part')
disp([theoreticalvalueexp;theoreticalvaluesec;theoreticalvaluepos])
disp('Estimated value: Expectation/2.Moment/pos. part')
disp([simulatedvalueexp;simulatedvaluesec;simulatedvaluepos])

%estimated variance
%estvarexp= var(CIR(end,:));
%estvarsec= var(CIR(end,:).^2);
%estvarpos= var(subplus(CIR(end,:)));
% confidence interval using CLT
%cfplusexp=simulatedvalueexp+1.96*sqrt(estvarexp/Nsimu);
%cfminusexp=simulatedvalueexp-1.96*sqrt(estvarexp/Nsimu);
%cfplussec=simulatedvaluesec+1.96*sqrt(estvarsec/Nsimu);
%cfminussec=simulatedvaluesec-1.96*sqrt(estvarsec/Nsimu);
%cfpluspos=simulatedvaluepos+1.96*sqrt(estvarpos/Nsimu);
%cfminuspos=simulatedvaluepos-1.96*sqrt(estvarpos/Nsimu);

%disp('Confidence interval: ')
%disp([cfminusexp,cfplusexp; cfminussec,cfplussec;cfminuspos,cfpluspos])

toc